import React, { ReactNode } from 'react';
import {
  ColorPaletteConfig,
  ExtraColorsForMui5,
  selectColorPaletteConfig,
  useAppSelector,
} from '../../data-access/store';
import { createTheme, ThemeOptions, ThemeProvider } from '@mui/material/styles';
import { ColorPalette } from './colorPaletteConfigSlice';

// Add custom theme variables to the mui theme types
declare module '@mui/material/styles' {
  interface Palette extends ColorPalette {}
  interface Theme {
    palette: Palette;
  }
  interface PaletteOptions extends Partial<ColorPalette> {}
  interface ThemeOptions {
    palette?: PaletteOptions;
  }
}

export function GraphPolarisThemeProvider({
  children,
}: {
  children: ReactNode;
}) {
  const colorPaletteConfig = useAppSelector(selectColorPaletteConfig);

  // Create a new theme when our custom color palette in redux changed
  const theme = React.useMemo(
    () =>
      // Map our color palette config (stored in redux) to the mui 5 format
      createTheme(MapColorsConfigToMuiTheme(colorPaletteConfig)),
    [colorPaletteConfig]
  );

  return <ThemeProvider theme={theme}>{children}</ThemeProvider>;
}

export function MapColorsConfigToMuiTheme(
  colorsConfig: ColorPaletteConfig
): ThemeOptions {
  return {
    palette: {
      mode: colorsConfig.darkMode ? 'dark' : 'light',
      ...(colorsConfig.darkMode
        ? colorsConfig.darkPalette
        : colorsConfig.lightPalette),
      // ...(colorsConfig.darkMode
      //   ? colorsConfig.darkPalette.custom
      //   : colorsConfig.lightPalette.custom) as ExtraColorsForMui5,
    },
  };
}

export default GraphPolarisThemeProvider;
