import {
  CheckDatatypeConstraint,
  GetAttributeBoolOperators,
} from '@graphpolaris/shared/lib/querybuilder/usecases';
import {
  updateQBAttributeOperator,
  updateQBAttributeValue,
  useAppDispatch,
} from '@graphpolaris/shared/lib/data-access/store';
import { useTheme } from '@mui/material';
import React, { useMemo, useState } from 'react';
import styles from './attributepill.module.scss';
import AttributeOperatorSelect from './operatorselect';
import { NodeProps } from 'reactflow';

/**
 * Component to render an attribute flow element
 * @param {FlowElement<EntityData>)} param0 The data of an entity flow element.
 */
export const AttributeRFPill = React.memo(({ id, data }: NodeProps) => {
  const theme = useTheme();
  const dispatch = useAppDispatch();
  const [value, setValue] = useState(data?.value || '');

  const onChange = (e: any) => {
    setValue(e.target.value);
  };
  const validateInput = () => {
    const newValue = CheckDatatypeConstraint(data.datatype, value);
    setValue(newValue);
    dispatch(updateQBAttributeValue({ id, value: newValue }));
  };

  // Calculates the size of the input
  const getInputWidth = () => {
    if (value == '') return 1;
    else if (value.length > 10) return 10;
    return value.length;
  };

  const boolOperators = useMemo(
    () => GetAttributeBoolOperators(data?.datatype),
    [data?.datatype]
  );

  // Determine the backgroundcolor based on if the attribute is connected to a entity or relation
  let bgcolor;
  if (data?.attributeOfA == 'entity')
    bgcolor = theme.palette.custom.queryBuilder.entity.lighterbg;
  else if (data?.attributeOfA == 'relation')
    bgcolor = theme.palette.custom.queryBuilder.relation.lighterbg;
  else bgcolor = theme.palette.custom.queryBuilder.attribute.background;

  return (
    <div
      className={styles.attribute}
      style={{
        background: bgcolor,
        color: theme.palette.custom.queryBuilder.text,
      }}
    >
      {/* <Handle
        id={Handles.Attribute}
        type="source"
        position={Position.Bottom}
        className={styles.handle}
      /> */}
      <div className={styles.contentWrapper}>
        <span className={styles.content} title={data.name}>
          {data.name}
        </span>
        <AttributeOperatorSelect
          selected={data?.operator}
          options={boolOperators}
          changed={(o) =>
            dispatch(updateQBAttributeOperator({ id, operator: o.value }))
          }
        />
        <span className={styles.attributeInput}>
          <input
            style={{ maxWidth: `${getInputWidth()}ch` }}
            type="string"
            placeholder={'?'}
            value={value}
            onChange={onChange}
            onBlur={validateInput}
            onKeyDown={(e) => e.key == 'Enter' && validateInput()}
          ></input>
        </span>
      </div>
    </div>
  );
});
AttributeRFPill.displayName = 'AttributeRFPill';

export default AttributeRFPill;
